﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Configuration;
using System.Xml;
using System.Collections;
using System.Collections.Specialized;
using SqlDeployTool.Properties;
using System.IO;
using System.Xml.Linq;
using SqlDeployConsole.Core;
using SqlDeployConsole;

namespace SqlDeployTool
{
    public partial class MainForm : Form
    {
        internal static XElement IncludedScriptXml = null;

        public MainForm()
        {
            InitializeComponent();
        }

        protected string configPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, Constants.ConfigurationFilePath);
        protected bool isWindowsAuthentication = false;
        protected XDocument xDocument = null;

        /// <summary>
        /// Handles the Load event of the MainForm control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            Bitmap bmp = Resources.Logo;
            this.Icon = Icon.FromHandle(bmp.GetHicon());

            this.rdBtnWindow.Checked = true;

            xDocument = XDocument.Load(configPath, LoadOptions.None);
            XElement connectionStringNode = xDocument.Root.Element(Constants.ConnectionString);
            ConnectionStringModel model = ConnectionStringModel.LoadConnectionString(connectionStringNode);
            SetControlValueByConnectionStringModel(model);
            isWindowsAuthentication = string.IsNullOrWhiteSpace(model.UserName);
            this.txtConnectionString.Text = GetConnectionStringByControl(isWindowsAuthentication);
            ChooseLogType(isWindowsAuthentication);

            InitializeVersionCombo(xDocument.Root.Elements(Constants.Increment));
        }

        /// <summary>
        /// Handles the Click event of the btnExit control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void btnExit_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        /// <summary>
        /// Handles the Click event of the btnImport control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void btnExecution_Click(object sender, EventArgs e)
        {
            string consolePath = string.Empty;
            consolePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "SqlDeployConsole.exe");

            System.Diagnostics.Process.Start(consolePath, GenerateCmdParameters());
        }

        /// <summary>
        /// Handles the Click event of the rdBtnWindow control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void rdBtnWindow_Click(object sender, EventArgs e)
        {
            isWindowsAuthentication = true;
            ChooseLogType(isWindowsAuthentication);
            this.txtConnectionString.Text = GetConnectionStringByControl(isWindowsAuthentication);
        }

        /// <summary>
        /// Handles the Click event of the rdBtnSQL control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void rdBtnSQL_Click(object sender, EventArgs e)
        {
            isWindowsAuthentication = false;
            ChooseLogType(isWindowsAuthentication);
            this.txtConnectionString.Text = GetConnectionStringByControl(isWindowsAuthentication);
        }

        /// <summary>
        /// Handles the Leave event of the txtConnectionString control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void txtConnectionString_Leave(object sender, EventArgs e)
        {
            ConnectionStringModel connectionStringModel = ConnectionStringModel.LoadConnectionString(this.txtConnectionString.Text);
            SetControlValueByConnectionStringModel(connectionStringModel);
        }

        /// <summary>
        /// Updates the connection string by value changed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void UpdateConnectionStringByValueChanged(object sender, EventArgs e)
        {
            ConnectionStringModel model = GetConnectionStringModelByControl();
            this.txtConnectionString.Text = GetConnectionStringByControl(isWindowsAuthentication);
        }


        /// <summary>
        /// Handles the Click event of the btn_TestConnection control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void btn_TestConnection_Click(object sender, EventArgs e)
        {
            bool isSuccess = false;

            string versionInfo = string.Empty;
            try
            {
                versionInfo = ScriptHelper.TestConnection(GetConnectionStringModelByControl().ToConnectionString());
                isSuccess = true;
            }
            catch (Exception ex)
            {
                versionInfo = ex.Message;
            }
            MessageBox.Show(versionInfo, isSuccess ? "Success" : "Failed", MessageBoxButtons.OK, isSuccess ? MessageBoxIcon.Information : MessageBoxIcon.Error);
        }

        /// <summary>
        /// Handles the Click event of the btn_Advanced control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void btn_Advanced_Click(object sender, EventArgs e)
        {
            AdvancedForm form = new AdvancedForm(IncludedScriptXml);
            form.ShowDialog();
        }

        /// <summary>
        /// Gets the connection string model by control.
        /// </summary>
        /// <param name="result">if set to <c>true</c> [result].</param>
        /// <returns></returns>
        private ConnectionStringModel GetConnectionStringModelByControl()
        {
            ConnectionStringModel model = new ConnectionStringModel();
            model.Server = this.txtServerIP.Text;
            model.DataBase = this.txtDataBase.Text;
            model.UserName = this.txtUID.Text;
            model.Password = this.txtPwd.Text;

            if (rdBtnWindow.Checked)
            {
                model.UserName = model.Password = string.Empty;
            }
            return model;
        }

        /// <summary>
        /// Sets the control value by connection string model.
        /// </summary>
        /// <param name="connectionStringModel">The connection string model.</param>
        private void SetControlValueByConnectionStringModel(ConnectionStringModel connectionStringModel)
        {
            this.txtServerIP.Text = connectionStringModel.Server;
            this.txtDataBase.Text = connectionStringModel.DataBase;
            this.txtUID.Text = connectionStringModel.UserName;
            this.txtPwd.Text = connectionStringModel.Password;
        }

        /// <summary>
        /// Gets the connection string by control.
        /// </summary>
        /// <param name="isWindowsAuthentication">if set to <c>true</c> [is windows authentication].</param>
        /// <returns></returns>
        private string GetConnectionStringByControl(bool isWindowsAuthentication)
        {
            ConnectionStringModel model = GetConnectionStringModelByControl();
            if (isWindowsAuthentication)
            {
                model.UserName = model.Password = string.Empty;
            }
            return model.ToConnectionString();
        }

        /// <summary>
        /// Handles the Click event of the btn_SaveCmdAs control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void btn_SaveCmdAs_Click(object sender, EventArgs e)
        {

        }


        /// <summary>
        /// Chooses the type of the log.
        /// </summary>
        /// <param name="isWindowsAuthentication">if set to <c>true</c> [is windows authentication].</param>
        private void ChooseLogType(bool isWindowsAuthentication)
        {
            this.rdBtnWindow.Checked = isWindowsAuthentication;
            this.rdBtnSQL.Checked = !isWindowsAuthentication;
            this.txtPwd.ReadOnly = this.txtUID.ReadOnly = isWindowsAuthentication;
        }

        /// <summary>
        /// Initializes the version combo.
        /// </summary>
        /// <param name="xElements">The x elements.</param>
        private void InitializeVersionCombo(IEnumerable<XElement> xElements)
        {
            const string format = "{0} ( {1} -> {2} )";
            this.combo_DBVerions.Items.Clear();
            XElement fullNode = this.xDocument.Root.Element(Constants.Full);
            this.combo_DBVerions.Items.Add(new ComboboxItem { Value = fullNode, Text = string.Format(format, "Full", "x", fullNode.Attribute(Constants.TargetVersion).Value) });

            foreach (var one in xElements)
            {
                this.combo_DBVerions.Items.Add(new ComboboxItem { Value = one, Text = string.Format(format, "Incremental", one.Attribute(Constants.BaseVersion).Value, one.Attribute(Constants.TargetVersion).Value) });
            }

            this.combo_DBVerions.SelectedIndex = 0;
        }

        /// <summary>
        /// Gets the selected version XML.
        /// </summary>
        /// <returns></returns>
        private XElement GetSelectedVersionXml()
        {
            ComboboxItem selectedVersion = this.combo_DBVerions.SelectedItem as ComboboxItem;
            return selectedVersion.Value as XElement;
        }

        /// <summary>
        /// Generates the CMD parameters.
        /// </summary>
        /// <returns></returns>
        private string GenerateCmdParameters()
        {
            return "\"" + GetConnectionStringModelByControl().ToBase64() + "\" \"" + StringToBase64((IncludedScriptXml).ToString()) + "\"";
        }

        /// <summary>
        /// Strings to base64.
        /// </summary>
        /// <param name="input">The input.</param>
        /// <returns></returns>
        public static string StringToBase64(string input)
        {
            try
            {
                byte[] bytes = System.Text.Encoding.UTF8.GetBytes(input);
                return Convert.ToBase64String(bytes);
            }
            catch
            {
                return string.Empty;
            }
        }

        private void combo_DBVerions_SelectedIndexChanged(object sender, EventArgs e)
        {
            IncludedScriptXml = GetSelectedVersionXml();
        }
    }

}
